@@ -1,7 +1,7 @@
 /*
    Kickshaw - A Menu Editor for Openbox
 
-   Copyright (c) 2010–2022        Marcus Schätzle
+   Copyright (c) 2010–2024        Marcus Schätzle
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -125,14 +125,14 @@ static gboolean closing_tags (const gboolean          filter_iteration_completed
 
         1 = option of action (='command' and 'prompt')
         <item label="Program">
-             <action="Exeucte">
+             <action="Execute">
                 <command>program</command>
             </action> = removed 1 in comparison to 'command'
         </item>
         <item="New item">
         ...
 
-        2 = startupnotify option (='enabled', 'name', 'wmclass' and 'icon')
+        2 = startupnotify option (='enabled', 'name', 'wmclass', and 'icon')
         <item label="Program">
             <action="Execute">
                  <command>program</command>
@@ -587,7 +587,7 @@ static void process_menu_or_item (GtkTreeIter      *process_iter,
 
     If a menu that contained invisible orphaned menus is saved with the "Keep Root Menu Separate in Saved Menu File" being disabled, 
     this filter callback function renders the children of such menus visible, provided that they have a label. 
-    If they don't have a label, child menus, pipe menus and items will remain invisible and are marked as such. The same applies 
+    If they don't have a label, child menus, pipe menus, and items will remain invisible and are marked as such. The same applies 
     if a former invisible orphaned menu didn't have a label; it is now no longer orphaned, but due to the missing label it 
     remains invisible.
 
@@ -618,11 +618,13 @@ static gboolean visualize_invisible_children (GtkTreeModel *filter_model,
         else { 
             gtk_tree_store_set (ks.treestore, &model_iter, TS_ELEMENT_VISIBILITY, 
                                 ((menu_element_txt) || STREQ (type_txt, "separator")) ? "visible" : 
-                                ((streq_any (type_txt, "menu", "pipe menu", NULL)) ? "invisible menu" : "invisible item"), -1);                
+                                ((streq_any (type_txt, "menu", "pipe menu", NULL)) ? "invisible menu" : 
+                                "invisible item"), 
+                                -1);                
         }
     }
 
-    return FALSE; // Continue iterating
+    return CONTINUE_ITERATING;
 }
 
 /* 
@@ -649,8 +651,8 @@ void save_menu (gchar *save_as_filename)
     g_autoptr(GFileOutputStream) menu_file_output_stream;
 
     if (G_UNLIKELY (!(menu_file_output_stream = g_file_replace (menu_file, NULL, ks.settings.create_backup, G_FILE_CREATE_NONE, NULL, &error)))) {
-        g_autofree gchar *err_msg = g_strdup_printf ("<b>An error occurred during the creation of the menu file</b> "
-                                                     "<tt>%s</tt> <b>:</b>\n%s", 
+        g_autofree gchar *err_msg = g_strdup_printf (_("<b>An error occurred during the creation of the menu file</b> "
+                                                       "<tt>%s</tt> <b>:</b>\n%s"), 
                                                      preliminary_filename, error->message);
 
         show_errmsg (err_msg);
@@ -658,7 +660,7 @@ void save_menu (gchar *save_as_filename)
 
     // The program will return here in case of an error during the writing process.
     if (G_UNLIKELY (setjmp (buf))) {
-        g_autofree gchar *err_msg = g_strdup_printf ("<b>An error occurred during the menu file writing process:</b>\n%s", 
+        g_autofree gchar *err_msg = g_strdup_printf (_("<b>An error occurred during the menu file writing process:</b>\n%s"), 
                                                      error->message);
 
         show_errmsg (err_msg);
@@ -823,28 +825,29 @@ void save_menu (gchar *save_as_filename)
         longjmp (buf, 1);
     }
 
-    g_autofree gchar *standard_file_path = g_strconcat (g_getenv ("HOME"), "/.config/openbox/menu.xml", NULL);
+    g_autofree gchar *standard_file_path = g_build_filename (ks.home_dir, ".config/openbox/menu.xml", NULL);
 
     if (STREQ (ks.filename, standard_file_path)) {
-        g_autofree gchar *standard_output;
+        g_autofree gchar *standard_output = NULL;
 
-        g_spawn_command_line_sync ("ps cax", &standard_output, NULL, NULL, &error);
-
-        if (G_UNLIKELY (error)) {
-            g_autofree gchar *err_msg = g_strdup_printf ("<b>Could not get a list of the running processes\n\nError:</b> %s", 
+        if (G_UNLIKELY (!g_spawn_command_line_sync ("ps cax", &standard_output, NULL, NULL, &error))) {
+            g_autofree gchar *err_msg = g_strdup_printf (_("<b>The menu has been saved, but it wasn't possible to obtain a list "
+                                                           "of the running processes to determine if Openbox is currently running.\n\n"
+                                                           "Error:</b> %s"), 
                                                          error->message);
 
             show_errmsg (err_msg);
         }
         else if (strstr (standard_output, "openbox")) {
-            gchar *args[] = { "openbox", "--reconfigure", NULL };
+            g_autofree gchar *standard_output = NULL;
 
-            g_spawn_async_with_pipes (NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL, NULL, NULL, &error);
+            g_spawn_command_line_sync ("openbox --reconfigure", &standard_output, NULL, NULL, &error);
 
-            if (G_UNLIKELY (error)) {
-                g_autofree gchar *err_msg = g_strdup_printf ("<b>The menu has been saved, but there was a problem "
-                                                             "with the reconfiguration of Openbox.\n\nError:</b> %s", 
-                                                             error->message);
+            // Error messages for openbox --reconfigure go to standard output.
+            if (G_UNLIKELY (*standard_output || error)) {
+                g_autofree gchar *err_msg = g_strdup_printf (_("<b>The menu has been saved, but there was a problem "
+                                                               "with the reconfiguration of Openbox.\n\nError:</b> %s"),
+                                                             (error) ? error->message : standard_output);
 
                 show_errmsg (err_msg);
             }
@@ -862,8 +865,8 @@ void save_menu (gchar *save_as_filename)
         while (valid) {
             get_field_values (save_txts_toplevel, ks.ts_model, &save_menu_iter);
 
-            if (streq_any (save_txts_toplevel[TYPE_TXT], "menu", "pipe menu", NULL) && 
-                STREQ (save_txts_toplevel[ELEMENT_VISIBILITY_TXT], "invisible orphaned menu")) {
+            if (G_UNLIKELY (streq_any (save_txts_toplevel[TYPE_TXT], "menu", "pipe menu", NULL) && 
+                            STREQ (save_txts_toplevel[ELEMENT_VISIBILITY_TXT], "invisible orphaned menu"))) {
                 gtk_tree_store_set (GTK_TREE_STORE (ks.ts_model), &save_menu_iter, TS_ELEMENT_VISIBILITY,
                                     (save_txts_toplevel[MENU_ELEMENT_TXT]) ? "visible" : "invisible menu", -1);
 
@@ -884,18 +887,18 @@ void save_menu (gchar *save_as_filename)
         }
 
         if (invisible_orphaned_menu_has_been_converted_to_visible) {
-            show_message_dialog ("Former Invisible Orphaned Menus Are Now Visible", "gtk-info", 
-                                 "This menu contained invisible orphaned menus. Since this menu has been saved "
-                                 "right now with the \"Keep Root Menu Separate in Saved Menu File\" option "
-                                 "being <b>disabled</b>, these menus have become part of the root menu now and, "
-                                 "<b>provided that they have a label</b>, are therefore no longer invisible.", 
-                                 "_Close");
+            show_message_dialog (_("Former Invisible Orphaned Menus Are Now Visible"), "gtk-info", 
+                                 _("This menu contained invisible orphaned menus. Since this menu has been saved "
+                                   "right now with the \"Keep Root Menu Separate in Saved Menu File\" option "
+                                   "being <b>disabled</b>, these menus have become part of the root menu now and, "
+                                   "<b>provided that they have a label</b>, are therefore no longer invisible."), 
+                                 _("_Close"));
         }
     }
 
-	/* If the file is saved to the standard path .config/openbox/menu.xml, Kickshaw will open this file 
-       automatically after a restart. It is thus not necessary to keep an autosave after saving in this case. */	
-    if (G_LIKELY (regex_match_simple (".config/openbox/menu.xml", ks.filename, PCRE2_UTF, PCRE2_ENDANCHORED))) {
+    /* If the file is saved to the standard path .config/openbox/menu.xml, Kickshaw will open this file 
+       automatically after a restart. It is thus not necessary to keep an autosave after saving in this case. */    
+    if (G_LIKELY (g_regex_match_simple (".*\\.config/openbox/menu\\.xml$", ks.filename, 0, 0))) {
         delete_autosave ();
     }
 
@@ -911,7 +914,7 @@ void save_menu (gchar *save_as_filename)
 
     if (G_UNLIKELY (!(STREQ (filename_prior_to_saving, ks.filename)))) {
         // Replace the current undo stack item with a new one that contains the new file name.
-        push_new_item_on_undo_stack (TRUE);
+        push_new_item_on_undo_stack (AFTER_SAVE);
     }
     // Change sensitivity of save button and remove additional save button tooltip texts, if they were shown.
     row_selected ();
